import boto3
from botocore.exceptions import ClientError
from cloudlift.exceptions import UnrecoverableException

from cloudlift.config import EnvironmentConfiguration
from cloudlift.config.logging import log_err

def get_region_for_environment(environment):
    if environment:
        return EnvironmentConfiguration(environment).get_config()[environment]['region']
    else:
        # Get the region from the AWS credentials used to execute cloudlift
        aws_session = boto3.session.Session()
        return aws_session.region_name


def get_client_for(resource, environment):
    try:
        return boto3.session.Session(
            region_name=get_region_for_environment(environment)
        ).client(resource)
    except ClientError as error:
        if error.response['Error']['Code'] == 'ExpiredTokenException':
            raise UnrecoverableException("AWS session associated with this profile has expired or is otherwise invalid")
        elif error.response['Error']['Code'] == 'InvalidIdentityTokenException':
            raise UnrecoverableException("AWS token that was passed could not be validated by Amazon Web Services")
        elif error.response['Error']['Code'] == 'RegionDisabledException':
            raise UnrecoverableException("STS is not activated in the requested region for the account that is being asked to generate credentials")
        elif error.response['Error']['Code'] == 'AccessDeniedException':
            raise UnrecoverableException("User is not authorized to perform get boto3 client session on %s" % resource)
        else:
            raise UnrecoverableException("Unable to find valid AWS credentials")

def get_resource_for(resource, environment):
    try:
        return boto3.session.Session(
            region_name=get_region_for_environment(environment)
        ).resource(resource)
    except ClientError as error:
        if error.response['Error']['Code'] == 'ExpiredTokenException':
            raise UnrecoverableException(
                "AWS session associated with this profile has expired or is otherwise invalid")
        elif error.response['Error']['Code'] == 'InvalidIdentityTokenException':
            raise UnrecoverableException(
                "AWS token that was passed could not be validated by Amazon Web Services")
        elif error.response['Error']['Code'] == 'RegionDisabledException':
            raise UnrecoverableException(
                "STS is not activated in the requested region for the account that is being asked to generate credentials")
        elif error.response['Error']['Code'] == 'AccessDeniedException':
            raise UnrecoverableException("User is not authorized to perform get boto3 resource session on %s" % resource)
        else:
            raise UnrecoverableException(
                "Unable to find valid AWS credentials")


def get_notifications_arn_for_environment(environment):
    try:
        return EnvironmentConfiguration(
            environment
        ).get_config()[environment]['environment']["notifications_arn"]
    except KeyError:
        raise UnrecoverableException("Unable to find notifications arn for {environment}".format(**locals()))


def get_ssl_certification_for_environment(environment):
    try:
        return EnvironmentConfiguration(
            environment
        ).get_config()[environment]['environment']["ssl_certificate_arn"]
    except KeyError:
        raise UnrecoverableException("Unable to find ssl certificate for {environment}".format(**locals()))



import boto3
from time import sleep
from cloudlift.config.logging import log_bold, log_warning, log


class DynamodbConfiguration:
    """
        Handles configuration in DynamoDB for cloudlift
    """

    def __init__(self, table_name, kv_pairs):
        session = boto3.session.Session()
        self.dynamodb = session.resource('dynamodb')
        self.dynamodb_client = session.client('dynamodb')
        self.kv_pairs = kv_pairs
        self.table_name = table_name

    def _get_table(self):
        table_names = self.dynamodb_client.list_tables()['TableNames']
        if self.table_name not in table_names:
            log_warning("Could not find {} table, creating one..".format(self.table_name))
            self._create_configuration_table()
            self._table_status()
        return self.dynamodb.Table(self.table_name)

    def _create_configuration_table(self):
        key_schema = [
            {'AttributeName': self.kv_pairs[0][0], 'KeyType': 'HASH'}]
        key_schema.extend([{'AttributeName': key, 'KeyType': 'RANGE'}
                            for key, _ in self.kv_pairs[1:]])
        self.dynamodb.create_table(
            TableName=self.table_name,
            KeySchema=key_schema,
            AttributeDefinitions=[
                {'AttributeName': key, 'AttributeType': 'S'} for key, _ in self.kv_pairs],
            BillingMode='PAY_PER_REQUEST'
        )
        log_bold("{} table created!".format(self.table_name))

    def _table_status(self):
        status = ""
        while status == "ACTIVE":
            log("Checking {} table status...".format(self.table_name))
            sleep(1)
            status = self.dynamodb_client.describe_table(
                TableName=self.table_name)["Table"]["TableStatus"]
        sleep(10)
        log("{} table status is ACTIVE".format(self.table_name))



import boto3
from botocore.exceptions import ClientError
from cloudlift.exceptions import UnrecoverableException
from cloudlift.config.logging import log_err


def check_sns_topic_exists(topic_name, environment):
    session = boto3.session.Session()
    sns_client = session.client('sns')
    try:
        sns_client.get_topic_attributes(TopicArn=topic_name)
        return True
    except ClientError as e:
        if e.response['Error']['Code'] == 'NotFound':
            raise UnrecoverableException(
                "Unable to find SNS topic {topic_name} in {environment} environment".format(**locals()))
        else:
            raise UnrecoverableException(e.response['Error']['Message'])
